import { EditorView, basicSetup } from 'codemirror';
import { formatKeymap } from './extensions/tab';
let editor: EditorView = null;

export type EditorOptions = {
  extensions?: any[];
  hasBasicSetup?: boolean;
};
/**
 * 创建单例
 * @param el
 * @returns
 */
const createEditorInstance = (el?: HTMLDivElement, opts?: EditorOptions) => {
  if (editor && el) {
    el.appendChild(editor.dom);
    return editor;
  } else if (editor) {
    return editor;
  }
  const extensions = opts?.extensions || [];
  extensions.push(formatKeymap);
  const hasBaseicSetup = opts?.hasBasicSetup ?? true;
  if (hasBaseicSetup) {
    extensions.unshift(basicSetup);
  }
  editor = new EditorView({
    extensions: extensions,
    parent: el || document.body,
  });
  editor.dom.style.height = '100%';
  return editor;
};

/**
 * 每次都创建新的实例
 * @param el
 * @returns
 */
export const createEditor = (el: HTMLDivElement, opts?: EditorOptions) => {
  const extensions = opts?.extensions || [];
  const hasBaseicSetup = opts?.hasBasicSetup ?? true;
  if (hasBaseicSetup) {
    extensions.unshift(basicSetup);
  }
  extensions.push(formatKeymap);
  const editor = new EditorView({
    extensions,
    parent: el || document.body,
  });
  editor.dom.style.height = '100%';
  return editor;
};
export const getEditor = () => editor;

export { editor, createEditorInstance };
